package de.howaner.Poketherus.entity;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.utils.Disposable;
import de.howaner.Poketherus.PokemonGame;
import de.howaner.Poketherus.map.MapTerrain;
import de.howaner.Poketherus.map.TileAnimation;
import de.howaner.Poketherus.map.renderer.WalkAnimatedTilesetTile;
import de.howaner.Poketherus.map.renderer.particle.ParticleRenderer;
import de.howaner.Poketherus.multiplayer.packets.out.PacketOutChangeFace;
import de.howaner.Poketherus.screens.GameScreen;
import de.howaner.Poketherus.util.Location;

public class Entity extends Sprite implements Disposable {
	private int entityID;
	private EntityFace face = EntityFace.SOUTH;
	private EntityState state = EntityState.NORMAL;

	// --- Walk things --- //
	private float walkState = -1f;
	private final Vector2 goFrom = new Vector2();
	private final Animation[] moveAnimations;
	private int currentMoveAnimation = 0;
	private int nextMoveAnimation = 1;
	private boolean checkedTileAnimations = false;

	public Entity(int entityID, Animation[] moveAnimations) {
		super(moveAnimations[0].getKeyFrame(0), 0, 0, 16, 24);
		this.entityID = entityID;
		this.moveAnimations = moveAnimations;
	}

	public int getEntityID() {
		return this.entityID;
	}

	public void setEntityID(int entityID) {
		this.entityID = entityID;
	}

	public EntityState getState() {
		return this.state;
	}

	public void setState(EntityState state) {
		if (this.state == state) {
			return;
		}

		this.state = state;
		this.setAnimation(state.getAnimationOrdinal());
	}

	/// --- Walk things --- ///

	@Override
	public void draw(Batch batch) {
		this.handleWalk(Gdx.graphics.getDeltaTime());
		super.draw(batch);
	}

	private void handleWalk(float deltaTime) {
		if (this.walkState == -1f || this.canStartWalk()) {
			return;
		}

		float changeX = this.face.getVectorX() * 16 * deltaTime * this.getSpeed();
		float changeY = this.face.getVectorY() * 16 * deltaTime * this.getSpeed();

		float newWalkState = this.walkState;
		if (changeX == 0f) {
			newWalkState += Math.abs(changeY);
		} else if (changeY == 0f) {
			newWalkState += Math.abs(changeX);
		}

		if (newWalkState >= 16f) {
			this.finishWalk();
			return;
		}

		if (!this.checkedTileAnimations && newWalkState > 8f) {
			this.playTilesetAnimation((int) (this.goFrom.x / 16 + this.face.getVectorX()), (int) (this.goFrom.y / 16 + this.face.getVectorY()), true);
			this.checkedTileAnimations = true;
		}

		this.walkState = newWalkState;
		this.setX(this.getX() + changeX);
		this.setY(this.getY() + changeY);

		// Update Animation
		if ((int) (this.walkState / 2) >= 4) {
			this.setAnimation(this.nextMoveAnimation + this.state.getAnimationOrdinal());
		}
	}

	public void finishWalk() {
		if (this.walkState == -1f) {
			return;
		}

		this.setX(this.goFrom.x + this.face.getVectorX() * 16);
		this.setY(this.goFrom.y + this.face.getVectorY() * 16);

		this.walkState = -1f;
		this.setAnimation(this.state.getAnimationOrdinal());
		this.nextMoveAnimation = Math.abs(this.nextMoveAnimation - 3);
		this.checkedTileAnimations = false;

		this.onFinishedWalk(WalkSource.WALK);
	}

	public boolean doStep(EntityFace face) {
		if (!this.canStartWalk()) {
			return false;
		}

		boolean hasChangedFace = (this.face != face);
		this.setFaceDirection(face);

		// Get new tile positions
		int tileX = this.getTileX() + this.face.getVectorX();
		int tileY = this.getTileY() + this.face.getVectorY();

		if (this.canMoveTo(tileX, tileY)) {
			this.setAnimation(this.state.getAnimationOrdinal());
			this.stopTilesetAnimation(this.getTileX(), this.getTileY());
			this.goFrom.set(this.getX(), this.getY());
			this.walkState = 0f;
			this.onStartWalk();
			return true;
		} else if (hasChangedFace) {
			if (this instanceof CameraPlayer && PokemonGame.getGameInstance().getMultiplayerHandler() != null) {
				PokemonGame.getGameInstance().getMultiplayerHandler().sendPacket(new PacketOutChangeFace(face));
			}
			return false;
		} else {
			return false;
		}
	}

	public void cancelWalk() {
		this.cancelWalk(true);
	}

	public void cancelWalk(boolean fixPosition) {
		if (this.walkState == -1f) {
			return;
		}

		if (fixPosition)
			this.setPosition(this.goFrom.x, this.goFrom.y);
		this.walkState = -1f;
		this.setAnimation(0);

		this.checkedTileAnimations = false;
	}

	public boolean canStartWalk() {
		return (this.walkState == -1f);
	}

	public int getAnimationID() {
		return this.currentMoveAnimation;
	}

	public void setAnimation(int id) {
		if (this.currentMoveAnimation == id) {
			return;
		}

		this.currentMoveAnimation = id;
		this.setRegion(this.moveAnimations[this.face.ordinal()].getKeyFrame(id));
	}

	public EntityFace getFaceDirection() {
		return this.face;
	}

	public void setFaceDirection(EntityFace face) {
		this.face = face;
		this.setRegion(this.moveAnimations[this.face.ordinal()].getKeyFrame(this.currentMoveAnimation));
	}

	public void playTilesetAnimation(int tileX, int tileY, boolean renderParticles) {
		MapTerrain map = GameScreen.getGameScreen().getCurrentWorld().getTerrain();
		for (int layer = map.getLayersNum() - 1; layer >= 0; layer--) {
			short tileId = map.getTileAt(tileX, tileY, layer);
			if (tileId == 0) {
				continue;
			}
			tileId--;

			TileAnimation tileAnimation = map.getTileset().getTileAnimation(tileId);
			if (tileAnimation == null) {
				continue;
			}

			if (tileAnimation.getAnimationType() != null && tileAnimation.getAnimationType() != TileAnimation.TileAnimationType.NORMAL) {
				((WalkAnimatedTilesetTile)map.getTileset().getRenderTile(tileId)).play(tileX, tileY);
			}

			if (!tileAnimation.getParticleName().isEmpty())
			{
				ParticleRenderer particleRenderer = GameScreen.getGameScreen().getParticleRenderer();
				particleRenderer.displayParticle(tileX * 16, tileY * 16, tileAnimation.getParticleDuration(), tileAnimation.getParticleName());
			}
		}
	}

	public void stopTilesetAnimation(int tileX, int tileY) {
		MapTerrain map = GameScreen.getGameScreen().getCurrentWorld().getTerrain();
		for (int layer = map.getLayersNum() - 1; layer >= 0; layer--) {
			short tileId = map.getTileAt(tileX, tileY, layer);
			if (tileId == 0) {
				continue;
			}
			tileId--;

			TileAnimation tileAnimation = map.getTileset().getTileAnimation(tileId);
			if (tileAnimation != null && tileAnimation.getAnimationType() == TileAnimation.TileAnimationType.STAND) {
				((WalkAnimatedTilesetTile)map.getTileset().getRenderTile(tileId)).stopAnimation(tileX, tileY);
			}
		}
	}

	public int getTileX() {
		return (int) (this.getX() / 16f);
	}

	public int getTileY() {
		return (int) (this.getY() / 16f);
	}

	public int getChunkX() {
		return this.getTileX() / 40;
	}

	public int getChunkY() {
		return this.getTileY() / 40;
	}

	public float getWalkState() {
		return this.walkState;
	}

	public Animation[] getMoveAnimations() {
		return this.moveAnimations;
	}

	public Location getLocation() {
		return new Location("", this.getTileX(), this.getTileY());
	}

	@Override
	public void dispose() {
		if (this.walkState != -1f) {
			this.walkState = -1f;
			this.stopTilesetAnimation(this.getTileX(), this.getTileY());
			this.setPosition(this.goFrom.x, this.goFrom.y);
		}

		this.stopTilesetAnimation(this.getTileX(), this.getTileY());
	}

	public enum WalkSource {
		WALK, JUMP;
	}

	/// --- API Things --- ///

	public float getSpeed() {
		switch (this.state) {
			case SPRINT:
				return 5.0f;
			case NORMAL:
				return 3.5f;
		}
		return 3.5f;
	}

	public boolean canMoveTo(int newX, int newY) {
		return true;
	}

	public void onFinishedWalk(WalkSource source) { }

	public void onStartWalk() { }

}
