package de.howaner.Poketherus.entity;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.math.Vector2;
import de.howaner.Poketherus.map.MapWalkData;
import de.howaner.Poketherus.resources.TextureManager;
import de.howaner.Poketherus.screens.GameScreen;
import de.howaner.Poketherus.world.World;

public class Player extends Entity {
	private String name;

	// --- Jump --- //
	private float jumpState = -1f;
	private final Vector2 jumpFrom = new Vector2(0, 0);
	private Sprite jumpShadowSprite;

	public Player(int entityID, String name) {
		super(entityID, TextureManager.PLAYER_MOVE_ANIMATIONS);
		this.name = name;
	}

	public String getName() {
		return this.name;
	}

	@Override
	public void draw(Batch batch) {
		super.draw(batch);

		// Draw jump shadow
		if (this.jumpState != -1 && this.jumpShadowSprite != null) {
			this.jumpShadowSprite.draw(batch);
		}

		// Handle jump
		this.handleJump(Gdx.graphics.getDeltaTime());
	}

	@Override
	public void dispose() {
		super.dispose();

		if (this.jumpState != -1f) {
			this.jumpState = -1f;
			this.stopTilesetAnimation(this.getTileX(), this.getTileY());
			this.setPosition(this.jumpFrom.x, this.jumpFrom.y);
		}

		this.stopTilesetAnimation(this.getTileX(), this.getTileY());
	}

	private void handleJump(float deltaTime) {
		if (!this.isInJump()) {
			return;
		}

		float changeX = this.getFaceDirection().getVectorX() * 16 * deltaTime * this.getSpeed() * 1.01f;
		float changeY = this.getFaceDirection().getVectorY() * 16 * deltaTime * this.getSpeed() * 1.01f;

		float newJumpState = this.jumpState;
		if (changeX == 0f) {
			newJumpState += Math.abs(changeY);
		} else if (changeY == 0f) {
			newJumpState += Math.abs(changeX);
		}

		this.jumpShadowSprite.setX(this.jumpShadowSprite.getX() + changeX);
		this.jumpShadowSprite.setY(this.jumpShadowSprite.getY() + changeY);

		// Update animation
		if (newJumpState < 16f) {
			this.setAnimation(2);
			this.setX(this.jumpShadowSprite.getX());
			this.setY(this.jumpShadowSprite.getY() + newJumpState * 1.5f);
		} else if (newJumpState < 32f) {
			this.setAnimation(1);
			this.setX(this.jumpShadowSprite.getX());
			this.setY(this.jumpShadowSprite.getY() + 24 - (newJumpState - 16) * 1.5f);
		} else {
			this.setX(this.jumpFrom.x + this.getFaceDirection().getVectorX() * 32);
			this.setY(this.jumpFrom.y + this.getFaceDirection().getVectorY() * 32);

			this.jumpState = -1f;
			this.jumpFrom.setZero();
			this.setAnimation(0);
			this.jumpShadowSprite = null;
			this.playTilesetAnimation(this.getTileX(), this.getTileY(), false);

			GameScreen.getGameScreen().getParticleRenderer().displayParticle(this.getX(), this.getY(), 0.14f, "dust");
			this.onFinishedWalk(WalkSource.JUMP);
			return;
		}

		this.jumpState = newJumpState;
	}
	
	@Override
	public boolean doStep(EntityFace face) {
		if (super.doStep(face)) {
			// Normal step
			return true;
		}
		if (!this.canStartWalk()) {
			// The player do already a step
			return false;
		}

		// Sets the new face direction
		this.setFaceDirection(face);

		// Get new tile positions
		int tileX = this.getTileX() + face.getVectorX();
		int tileY = this.getTileY() + face.getVectorY();

		World world = GameScreen.getGameScreen().getCurrentWorld();
		MapWalkData.WalkDataType walkData = world.getWalkdataAt(tileX, tileY);

		if (walkData.getAttachedFace() == face) {
			this.doJump();
			return true;
		}
		return false;
	}

	public boolean doJump() {
		if (!this.canStartWalk()) {
			return false;
		}

		this.jumpShadowSprite = new Sprite(TextureManager.JUMP_SHADOW, TextureManager.JUMP_SHADOW.getWidth(), TextureManager.JUMP_SHADOW.getHeight());
		this.jumpShadowSprite.setPosition(this.getX(), this.getY());

		this.stopTilesetAnimation(this.getTileX(), this.getTileY());
		this.jumpFrom.set(this.getX(), this.getY());
		this.jumpState = 0f;
		return true;
	}

	@Override
	public boolean canMoveTo(int newX, int newY) {
		World world = GameScreen.getGameScreen().getCurrentWorld();

		MapWalkData.WalkDataType oldWalkData = world.getWalkdataAt(this.getTileX(), this.getTileY());
		MapWalkData.WalkDataType newWalkData = world.getWalkdataAt(newX, newY);

		switch (newWalkData) {
			case NONE:
			case Layer0:
				return true;
			case BLOCKED:
				return false;
			case ONLY_NORTH:
			case ONLY_EAST:
			case ONLY_SOUTH:
			case ONLY_WEST:
				return false;
			case Layer1:
			case Layer2:
			case Layer3:
			case Layer4:
				return (oldWalkData == newWalkData || oldWalkData == MapWalkData.WalkDataType.Layer0 || oldWalkData == MapWalkData.WalkDataType.NONE);
			default:
				return false;
		}
	}

	@Override
	public boolean canStartWalk() {
		return (super.canStartWalk() && !this.isInJump());
	}

	public Sprite getShadowSprite() {
		return this.jumpShadowSprite;
	}

	public boolean isInJump() {
		return (this.jumpState != -1f);
	}

	@Override
	public float getSpeed() {
		if (this.isInJump()) {
			// Return jump speed
			return 3.0f;
		}

		return super.getSpeed();
	}

	@Override
	public void onFinishedWalk(WalkSource source) {
		super.onFinishedWalk(source);
	}

}
