package de.howaner.Poketherus.map;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.utils.Disposable;
import de.howaner.Poketherus.map.events.Event;
import de.howaner.Poketherus.map.events.MapEvents;
import de.howaner.Poketherus.saveformat.TagBase;
import de.howaner.Poketherus.saveformat.TagCompound;
import de.howaner.Poketherus.saveformat.TagList;
import de.howaner.Poketherus.saveformat.TagManager;
import java.util.ArrayList;
import java.util.List;

public class MapTerrainMainWorld extends MapTerrain {
	private int width;
	private int height;
	private int layersNum;
	private MapPart[] parts;

	@Override
	public String getName() {
		return "Hauptwelt";
	}

	@Override
	public String getJoinMessage() {
		return "";
	}

	@Override
	public int getWidth() {
		return this.width * 40;
	}

	@Override
	public int getHeight() {
		return this.height * 40;
	}

	public void unloadPart(int x, int y) {
		MapPart part = this.parts[y * this.width + x];
		if (part != null) {
			part.dispose();
		}
		this.parts[y * this.width + x] = null;
	}

	public List<MapPart> getLoadedParts() {
		List<MapPart> partList = new ArrayList<MapPart>();
		for (MapPart part : this.parts) {
			if (part != null) {
				partList.add(part);
			}
		}

		return partList;
	}

	public MapPart getPartAt(int x, int y) {
		int partX = x / 40;
		int partY = y / 40;

		if (partX >= this.width || partY >= this.height || partX < 0 || partY < 0) {
			return null;
		}
		return this.parts[partY * this.width + partX];
	}

	public String getJoinMessageAt(int x, int y) {
		MapPart part = this.getPartAt(x, y);
		if (part == null) {
			return "";
		}

		return part.joinMessage;
	}

	@Override
	public short getTileAt(int x, int y, int layer) {
		MapPart part = this.getPartAt(x, y);
		if (part == null) {
			return 0;
		}

		if (layer < 0 || layer >= part.layers.size()) {
			return 0;
		}

		return part.layers.get(layer).getTile(part.getPartX(x), part.getPartY(y));
	}

	@Override
	public MapWalkData.WalkDataType getWalkdataAt(int x, int y) {
		MapPart part = this.getPartAt(x, y);
		if (part == null) {
			return MapWalkData.WalkDataType.BLOCKED;
		}

		return part.walkData.getWalkData(part.getPartX(x), part.getPartY(y));
	}

	@Override
	public Event getEventAt(int x, int y) {
		MapPart part = this.getPartAt(x, y);
		if (part == null) {
			return null;
		}

		return part.events.getEventAt(part.getPartX(x), part.getPartY(y));
	}

	@Override
	public int getLayersNum() {
		return this.layersNum;
	}

	public void loadPart(int x, int y) {
		if (this.getPartAt(x, y) != null) {
			return;
		}

		try {
			String filePath = String.format("maps/Hauptwelt/Hauptwelt.%d.%d.pkmap", x, y);
			FileHandle file = Gdx.files.internal(filePath);
			if (!file.exists()) {
				// No chunk file for this chunk
				return;
			}

			TagCompound compound = TagManager.readCompressed(file.read());
			MapPart part = new MapPart();

			// Load options
			part.x = x;
			part.y = y;
			if (compound.containsTag("joinMessage"))
				part.joinMessage = compound.getString("joinMessage");

			// Load layers
			List<TagBase> layersList = compound.getList("layers");
			for (TagBase layerTag : layersList) {
				MapLayer layer = new MapLayer(part.layers.size(), 40, 40);
				layer.loadFromTagList(((TagList) layerTag).getData());
				part.layers.add(layer);
			}

			// Load walkdata
			part.walkData = new MapWalkData(40, 40);
			part.walkData.loadFromTagList(compound.getList("walkData"));

			// Load entities
			if (compound.containsTag("events")) {
				part.events = new MapEvents(this);
				part.events.loadFromTagList(compound.getList("events"));
			}

			this.parts[y * this.width + x] = part;
		} catch (Exception ex) {
			Gdx.app.error("ChunkLoader", "Can't load map part. (X: " + x + ", Y: " + y + ")", ex);
		}
	}

	@Override
	public void loadFromCompound(TagCompound compound) {
		this.width = compound.getInt("width");
		this.height = compound.getInt("height");
		this.layersNum = compound.getInt("layersNum");

		MapTileset tileset = new MapTileset();
		tileset.loadFromCompound(compound.getCompound("tileset"));
		this.setTileset(tileset);

		this.parts = new MapPart[this.width * this.height];
	}

	@Override
	public void dispose() {
		for (int i = 0; i < this.parts.length; i++) {
			MapPart part = this.parts[i];
			if (part != null)
				part.dispose();
			this.parts[i] = null;
		}
		this.parts = null;
	}

	public class MapPart implements Disposable {
		public int x, y;
		public String joinMessage = "";
		public List<MapLayer> layers = new ArrayList<MapLayer>();
		public MapWalkData walkData;
		public MapEvents events;

		public int getPartX(int globalX) {
			return globalX - x * 40;
		}

		public int getPartY(int globalY) {
			return globalY - y * 40;
		}

		@Override
		public void dispose() {
			this.walkData.dispose();
			this.events.dispose();

			for (MapLayer layer : this.layers) {
				layer.dispose();
			}
			this.layers.clear();

			this.walkData = null;
			this.events = null;
			this.layers = null;
			this.joinMessage = null;
		}
	}

}
