package de.howaner.Poketherus.map;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Gdx2DPixmap;
import com.badlogic.gdx.utils.Disposable;
import de.howaner.Poketherus.map.TileAnimation.TileAnimationType;
import de.howaner.Poketherus.map.renderer.AnimatedTilesetTile;
import de.howaner.Poketherus.map.renderer.WalkAnimatedTilesetTile;
import de.howaner.Poketherus.map.renderer.StaticTilesetTile;
import de.howaner.Poketherus.map.renderer.TilesetTile;
import de.howaner.Poketherus.saveformat.TagBase;
import de.howaner.Poketherus.saveformat.TagCompound;
import de.howaner.Poketherus.saveformat.TagString;
import de.howaner.Poketherus.util.Base64;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

public class MapTileset implements Disposable {
	private List<Texture> tiles = new ArrayList<Texture>();
	private TileAnimation[] animations;
	private final List<TilesetTile> renderTiles = new ArrayList<TilesetTile>();

	public MapTileset() {
	}

	public MapTileset(List<Texture> tiles) {
		this.tiles = tiles;
	}

	public List<Texture> getTiles() {
		return this.tiles;
	}

	public void setTiles(List<Texture> tiles) {
		this.tiles = tiles;
	}

	public void addTile(Texture tile) {
		this.tiles.add(tile);
	}

	public void removeTile(Texture tile) {
		this.tiles.remove(tile);
	}

	public Texture getTileFromId(int id) {
		if (id < 0 || id >= tiles.size()) {
			return null;
		}

		return this.tiles.get(id);
	}

	public void setTile(int id, Texture tile) {
		this.tiles.set(id, tile);
	}

	public TileAnimation getTileAnimation(short tile) {
		if (tile < 0 || tile >= this.animations.length) {
			return null;
		}
		return this.animations[tile];
	}

	public void setTileAnimation(short tile, TileAnimation animation) {
		if (tile < 0 || tile >= this.tiles.size()) {
			return;
		}

		this.animations[tile] = animation;
	}

	public void loadRenderTiles() {
		this.renderTiles.clear();

		for (short i = 0; i < this.tiles.size(); i++) {
			Texture texture = this.tiles.get(i);
			TileAnimation animation = this.getTileAnimation(i);
			TileAnimationType type = (animation == null) ? null : animation.getAnimationType();

			if (type == null) {
				this.renderTiles.add(new StaticTilesetTile(this, texture, i));
			} else if (type == TileAnimationType.NORMAL) {
				this.renderTiles.add(new AnimatedTilesetTile(this, texture, animation, i));
			} else {
				this.renderTiles.add(new WalkAnimatedTilesetTile(this, texture, animation, i));
			}
		}
	}

	public TilesetTile getRenderTile(short tileId) {
		if (tileId < 0 || tileId >= this.renderTiles.size()) {
			return null;
		}
		return this.renderTiles.get(tileId);
	}

	public void loadFromCompound(TagCompound compound) {
		this.tiles.clear();
		List<TagBase> tilesList = compound.getList("tiles");
		for (TagBase tag : tilesList) {
			if (!(tag instanceof TagString)) {
				continue;
			}

			String base64String = ((TagString) tag).getData();
			try {
				byte[] bytes = Base64.decode(base64String, Base64.DEFAULT);
				InputStream inputStream = new ByteArrayInputStream(bytes);

				Gdx2DPixmap pixmap = new Gdx2DPixmap(inputStream, Gdx2DPixmap.GDX2D_FORMAT_RGBA8888);
				this.addTile(new Texture(new Pixmap(pixmap)));
			} catch (Exception ex) {
				Gdx.app.error("Tileset", "Can't load tileset!", ex);
			}
		}

		this.animations = new TileAnimation[this.tiles.size()];

		if (compound.containsTag("animations")) {
			TagCompound animationsCompound = compound.getCompound("animations");
			for (String key : animationsCompound.getKeys()) {
				short id = Short.parseShort(key);
				TileAnimation animation = new TileAnimation();

				try {
					animation.loadFromCompound(animationsCompound.getCompound(key));
				} catch (Exception ex) {
					Gdx.app.error("Tileset", "Can't load tileset animation #" + id, ex);
					continue;
				}

				this.setTileAnimation(id, animation);
			}
		}

		this.loadRenderTiles();
	}

	@Override
	public void dispose() {
		for (Texture texture : this.tiles) {
			texture.dispose();
		}
		this.tiles.clear();

		for (int i = 0; i < this.animations.length; i++) {
			TileAnimation animation = this.animations[i];
			if (animation != null)
				animation.dispose();
			this.animations[i] = null;
		}
		this.animations = null;

		this.renderTiles.clear();
	}

}
