package de.howaner.Poketherus.map.renderer;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.g2d.Batch;
import static com.badlogic.gdx.graphics.g2d.Batch.*;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.maps.MapRenderer;
import com.badlogic.gdx.math.Matrix4;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.utils.Disposable;
import de.howaner.Poketherus.map.MapTerrain;

public class PkMapRenderer implements MapRenderer, Disposable {

	private MapTerrain map;
	private float unitScale;
	private Batch spriteBatch;
	private Rectangle viewBounds;
	private boolean ownsSpriteBatch;
	private float vertices[] = new float[20];

	public PkMapRenderer(MapTerrain map) {
		this(map, 1.0f);
	}

	public PkMapRenderer(MapTerrain map, float unitScale) {
		this.map = map;
		this.unitScale = unitScale;
		this.viewBounds = new Rectangle();
		this.spriteBatch = new SpriteBatch();
		this.ownsSpriteBatch = true;
	}

	public PkMapRenderer(MapTerrain map, Batch batch) {
		this(map, 1.0f, batch);
	}

	public PkMapRenderer(MapTerrain map, float unitScale, Batch batch) {
		this.map = map;
		this.unitScale = unitScale;
		this.viewBounds = new Rectangle();
		this.spriteBatch = batch;
		this.ownsSpriteBatch = false;
	}

	public MapTerrain getMap() {
		return this.map;
	}

	public void setMap(MapTerrain map) {
		this.map = map;
	}

	public float getUnitScale() {
		return this.unitScale;
	}

	public Batch getSpriteBatch() {
		return this.spriteBatch;
	}

	public Rectangle getViewBounds() {
		return this.viewBounds;
	}

	@Override
	public void setView(OrthographicCamera camera) {
		spriteBatch.setProjectionMatrix(camera.combined);
		float width = camera.viewportWidth * camera.zoom;
		float height = camera.viewportHeight * camera.zoom;
		this.viewBounds.set(camera.position.x - width / 2, camera.position.y - height / 2, width, height);
	}

	@Override
	public void setView(Matrix4 projection, float x, float y, float width, float height) {
		spriteBatch.setProjectionMatrix(projection);
		viewBounds.set(x, y, width, height);
	}

	@Override
	public void render() {
		this.beginRender();
		for (int layerID = 0; layerID < map.getLayersNum(); layerID++) {
			this.renderTileLayer(layerID);
		}
		this.endRender();
	}

	@Override
	public void render(int[] layers) {
		this.beginRender();
		for (int layerID : layers) {
			this.renderTileLayer(layerID);
		}
		this.endRender();
	}

	public void renderTileLayer(int layerID) {
		final Color batchColor = spriteBatch.getColor();
		final float color = Color.toFloatBits(batchColor.r, batchColor.g, batchColor.b, batchColor.a);

		final int layerWidth = this.map.getWidth();
		final int layerHeight = this.map.getHeight();

		final float layerTileWidth = 16 * unitScale;
		final float layerTileHeight = 16 * unitScale;

		final int col1 = Math.max(0, (int) (viewBounds.x / layerTileWidth));
		final int col2 = Math.min(layerWidth, (int) ((viewBounds.x + viewBounds.width + layerTileWidth) / layerTileWidth));

		final int row1 = Math.max(0, (int) (viewBounds.y / layerTileHeight));
		final int row2 = Math.min(layerHeight, (int) ((viewBounds.y + viewBounds.height + layerTileHeight) / layerTileHeight));

		float y = row2 * layerTileHeight;
		float xStart = col1 * layerTileWidth;
		final float[] vertices = this.vertices;

		for (int row = row2; row >= row1; row--) {
			float x = xStart;
			for (int col = col1; col < col2; col++) {
				short tile = this.map.getTileAt(col, row, layerID);
				if (tile == 0) {
					x += layerTileWidth;
					continue;
				}
				tile--;
				
				TilesetTile renderTile = this.map.getTileset().getRenderTile(tile);
				if (renderTile != null) {
					TextureRegion region = renderTile.getTextureRegion(col, row);

					float x2 = x + region.getRegionWidth() * unitScale;
					float y2 = y + region.getRegionHeight() * unitScale;

					float u1 = region.getU();
					float v1 = region.getV2();
					float u2 = region.getU2();
					float v2 = region.getV();

					vertices[X1] = x;
					vertices[Y1] = y;
					vertices[C1] = color;
					vertices[U1] = u1;
					vertices[V1] = v1;

					vertices[X2] = x;
					vertices[Y2] = y2;
					vertices[C2] = color;
					vertices[U2] = u1;
					vertices[V2] = v2;

					vertices[X3] = x2;
					vertices[Y3] = y2;
					vertices[C3] = color;
					vertices[U3] = u2;
					vertices[V3] = v2;

					vertices[X4] = x2;
					vertices[Y4] = y;
					vertices[C4] = color;
					vertices[U4] = u2;
					vertices[V4] = v1;

					spriteBatch.draw(region.getTexture(), vertices, 0, 20);
				}
				x += layerTileWidth;
			}
			y -= layerTileHeight;
		}
	}

	/**
	 * Called before the rendering of all layers starts.
	 */
	protected void beginRender() {
		// AnimatedTiledMapTile.updateAnimationBaseTime();
		spriteBatch.begin();
	}

	/**
	 * Called after the rendering of all layers ended.
	 */
	protected void endRender() {
		spriteBatch.end();
	}

	@Override
	public void dispose() {
		if (ownsSpriteBatch) {
			spriteBatch.dispose();
		}
	}

}
