package de.howaner.Poketherus.screens;

import aurelienribon.tweenengine.BaseTween;
import aurelienribon.tweenengine.Tween;
import aurelienribon.tweenengine.TweenCallback;
import aurelienribon.tweenengine.TweenManager;
import com.badlogic.gdx.Application.ApplicationType;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import de.howaner.Poketherus.PokemonGame;
import de.howaner.Poketherus.input.GameInput;
import de.howaner.Poketherus.entity.CameraPlayer;
import de.howaner.Poketherus.entity.Entity;
import de.howaner.Poketherus.gui.DebugRenderer;
import de.howaner.Poketherus.map.MapTerrainMainWorld;
import de.howaner.Poketherus.map.MapTerrain;
import de.howaner.Poketherus.map.MapTerrainMainWorld.MapPart;
import de.howaner.Poketherus.map.renderer.PkMapRenderer;
import de.howaner.Poketherus.map.renderer.particle.ParticleRenderer;
import de.howaner.Poketherus.saveformat.TagCompound;
import de.howaner.Poketherus.saveformat.TagManager;
import de.howaner.Poketherus.tween.SpriteAccessor;
import de.howaner.Poketherus.tween.SpriteBatchAccessor;
import de.howaner.Poketherus.util.MathHelper;
import de.howaner.Poketherus.world.World;
import java.util.List;

public class GameScreen implements Screen {
	private GameInput playerInput;
	private World currentWorld;
	private World mainWorld;
	private Thread chunkLoaderThread = null;
	private final CameraPlayer player;

	private ParticleRenderer particleRenderer;
	private DebugRenderer debugRenderer;
	private PkMapRenderer renderer;
	private OrthographicCamera camera;
	private TweenManager tweenManager;

	public GameScreen(CameraPlayer defaultPlayer) {
		this.player = defaultPlayer;
	}

	public static GameScreen getGameScreen() {
		return (GameScreen) PokemonGame.getGameInstance().getScreen();
	}

	public World getCurrentWorld() {
		return this.currentWorld;
	}

	public MapTerrain loadMap(String name) {
		FileHandle file = Gdx.files.internal("maps/" + name + ".pkmap");
		if (!file.exists()) {
			Gdx.app.error("MapLoader", "Unknown map: " + name);
			return null;
		}

		try {
			TagCompound compound = TagManager.readCompressed(file.read());

			MapTerrain map = new MapTerrain();
			map.setName(name);
			map.loadFromCompound(compound);
			Gdx.app.log("MapLoader", "Loaded map " + name + "!");
			return map;
		} catch (Exception ex) {
			Gdx.app.error("MapLoader", "Can't load map " + name, ex);
			return null;
		}
	}

	public void loadMainWorld() {
		if (this.mainWorld != null) {
			return;
		}

		try {
			TagCompound compound = TagManager.readFromInternalFile("maps/Hauptwelt/Hauptwelt.pkmap");

			MapTerrainMainWorld terrain = new MapTerrainMainWorld();
			terrain.loadFromCompound(compound);

			this.mainWorld = new World(terrain);
		} catch (Exception ex) {
			Gdx.app.error("MapLoader", "Can't load main world!", ex);
			Gdx.app.exit();
		}
	}

	public boolean switchToWorld(String worldName) {
		if (this.currentWorld != null && this.currentWorld.getName().equals(worldName)) {
			return true;
		}

		if (this.currentWorld != null) {
			this.currentWorld.removeAllEntities();
		}

		if (worldName.equals("Hauptwelt")) {
			if (this.currentWorld != null && this.currentWorld != this.mainWorld) {
				this.currentWorld.getTerrain().dispose();
			}

			this.currentWorld = this.mainWorld;
			if (this.renderer != null) {
				this.renderer.setMap(this.currentWorld.getTerrain());
			}
			return true;
		}

		MapTerrain terrain = this.loadMap(worldName);
		if (terrain == null) {
			return false;
		}

		if (this.currentWorld != this.mainWorld) {
			this.currentWorld.getTerrain().dispose();
		}

		this.currentWorld = new World(terrain);
		if (this.renderer != null)
			this.renderer.setMap(this.currentWorld.getTerrain());

		return true;
	}

	@Override
	public void render(float delta) {
		Gdx.gl20.glClearColor(0, 0, 0, 1);
		Gdx.gl20.glClear(GL20.GL_COLOR_BUFFER_BIT);

		this.tweenManager.update(delta);

		this.player.setCameraPosition(this.camera);
		this.camera.update();

		this.renderer.setView(this.camera);
		this.renderer.render(new int[] { 0, 1, 2 });

		this.renderer.getSpriteBatch().begin();
		for (Entity entity : this.currentWorld.getEntities()) {
			if (entity != this.player)
				entity.draw(this.renderer.getSpriteBatch());
		}
		this.player.draw(this.renderer.getSpriteBatch());
		this.particleRenderer.renderParticles(this.renderer.getSpriteBatch());
		this.renderer.getSpriteBatch().end();

		this.renderer.render(new int[] { 3, 4, 5, 6 });

		this.debugRenderer.draw(this.renderer.getSpriteBatch());
	}

	@Override
	public void resize(int width, int height) {
		float newWidth, newHeight;
		float maxWidth, maxHeight;

		if (Gdx.app.getType() == ApplicationType.Android) {
			newWidth = (float)width / 4f;
			newHeight = (float)height / 4f;
			maxWidth = 320f;
			maxHeight = 180f;
		} else {
			newWidth = (float)width / 3.25f;
			newHeight = (float)height / 3.25f;
			maxWidth = 393.85f;
			maxHeight = 221.54f;
		}

		if (newWidth > maxWidth) {
			newHeight = maxWidth * (newHeight / newWidth);
			newWidth = maxWidth;
		}
		if (newHeight > maxHeight) {
			newWidth = maxHeight * (newWidth / newHeight);
			newHeight = maxHeight;
		}

		camera.viewportWidth = newWidth;
		camera.viewportHeight = newHeight;
	}

	public void updateChunks(boolean async) {
		if (this.currentWorld != this.mainWorld) {
			return;
		}

		if (async) {
			if (this.chunkLoaderThread != null) {
				this.chunkLoaderThread.interrupt();
			}

			Thread thread = new Thread(new Runnable() {
				@Override
				public void run() {
					GameScreen.this.updateChunks(false);
					GameScreen.this.chunkLoaderThread = null;
				}
			});
			thread.setName("AsyncChunkLoader");
			this.chunkLoaderThread = thread;
			thread.start();
			return;
		}

		int[][] directions = new int[][] {
			{ 0, 0 },
			{ 0, 1 },
			{ 0, -1 },
			{ 1, 0 },
			{ 1, 1 },
			{ 1, -1 },
			{ -1, 0 },
			{ -1, 1 },
			{ -1, -1 },
		};

		MapTerrainMainWorld terrain = (MapTerrainMainWorld) this.mainWorld.getTerrain();
		int playerX = this.player.getChunkX();
		int playerY = this.player.getChunkY();
		for (int[] direction : directions) {
			int chunkX = playerX + direction[0];
			int chunkY = playerY + direction[1];

			terrain.loadPart(chunkX, chunkY);
		}

		// Unload old parts
		List<MapPart> loadedParts = terrain.getLoadedParts();
		for (MapPart part : loadedParts) {
			if (MathHelper.Diff(playerX, part.x) > 1 || MathHelper.Diff(playerY, part.y) > 1) {
				terrain.unloadPart(part.x, part.y);
			}
		}

		Gdx.app.log("ChunkLoader", "Updated chunks!");
	}

	@Override
	public void show() {
		this.camera = new OrthographicCamera(320, 180);
		this.renderer = new PkMapRenderer(this.currentWorld.getTerrain());
		this.particleRenderer = new ParticleRenderer();
		this.debugRenderer = new DebugRenderer(this);

		// Initialize tween
		this.tweenManager = new TweenManager();
		Tween.registerAccessor(Sprite.class, new SpriteAccessor());
		Tween.registerAccessor(SpriteBatch.class, new SpriteBatchAccessor());

		this.player.updateLastChunk();
		this.currentWorld.spawnEntity(this.player);
		this.updateChunks(false);

		this.playerInput = new GameInput(this);
		Gdx.input.setInputProcessor(this.playerInput);
		Gdx.app.log("Main", "Finished start!");
	}

	public CameraPlayer getPlayer() {
		return this.player;
	}

	public ParticleRenderer getParticleRenderer() {
		return this.particleRenderer;
	}

	public PkMapRenderer getMapRenderer() {
		return this.renderer;
	}

	public GameInput getPlayerInput() {
		return this.playerInput;
	}

	public DebugRenderer getDebugRenderer() {
		return this.debugRenderer;
	}

	@Override
	public void hide() {
		this.dispose();
	}

	@Override
	public void pause() {
		
	}

	@Override
	public void resume() {
		
	}

	@Override
	public void dispose() {
		Gdx.input.setInputProcessor(null);

		this.player.dispose();
		this.renderer.dispose();
		this.particleRenderer.dispose();
		this.debugRenderer.dispose();

		this.currentWorld.getTileset().dispose();
		if (this.currentWorld != this.mainWorld) {
			this.mainWorld.getTileset().dispose();
		}

		this.tweenManager.killAll();
	}

	/**
	 * Fade out the screen.
	 * @param duration The duration of the fade out. Standard is 0.5f
	 * @param fadedOut The runnable that will be called when the fade out finished.
	 */
	public void fadeOut(float duration, final Runnable fadedOut) {
		final Batch batch = this.getMapRenderer().getSpriteBatch();

		Tween.set(batch, SpriteBatchAccessor.ALPHA).target(1.0f).start(this.tweenManager);
		Tween.to(batch, SpriteBatchAccessor.ALPHA, duration).target(0.0f).setCallback(new TweenCallback() {
			@Override
			public void onEvent(int type, BaseTween<?> source) {
				if (fadedOut != null)
					fadedOut.run();
			}
		}).start(this.tweenManager);
	}

	public void fadeIn(float duration, final Runnable fadedIn) {
		final Batch batch = this.getMapRenderer().getSpriteBatch();

		Tween.set(batch, SpriteBatchAccessor.ALPHA).target(0.0f).start(this.tweenManager);
		Tween.to(batch, SpriteBatchAccessor.ALPHA, duration).target(1f).setCallback(new TweenCallback() {
			@Override
			public void onEvent(int type, BaseTween<?> source) {
				if (fadedIn != null)
					fadedIn.run();
			}
		}).start(this.tweenManager);
	}

	public void fadeEntityOut(int entityID, float duration) {
		Entity entity = this.currentWorld.getEntity(entityID);
		if (entity == null) {
			return;
		}

		Tween.set(entity, SpriteAccessor.ALPHA).target(1.0f).start(this.tweenManager);
		Tween.to(entity, SpriteAccessor.ALPHA, duration).target(0.0f).start(this.tweenManager);
	}

	public void fadeEntityIn(int entityID, float duration) {
		Entity entity = this.currentWorld.getEntity(entityID);
		if (entity == null) {
			return;
		}

		Tween.set(entity, SpriteAccessor.ALPHA).target(0.0f).start(this.tweenManager);
		Tween.to(entity, SpriteAccessor.ALPHA, duration).target(1f).start(this.tweenManager);
	}

}
